let buffer = 2048;
let context;
let processor;
let input;
let globalStream;
let audioData = [];

const startRecordButton = document.getElementById("start-record");
const stopRecordButton = document.getElementById("stop-record");
const recordProgress = document.getElementById("record-progress");

// 録音を開始する
function startRecord(element) {
  showSpinner(element);
  startRecordButton.disabled = true;
  stopRecordButton.disabled = false;
  record();
  recordProgress.innerText = "録音中..."
}

function record() {
  context = new AudioContext();
  processor = context.createScriptProcessor(buffer, 1, 1);
  processor.connect(context.destination);
  context.resume();

  let recording = (stream) => {
    globalStream = stream;
    input = context.createMediaStreamSource(stream);
    input.connect(processor);
    processor.onaudioprocess = (audio) => {
      let input = audio.inputBuffer.getChannelData(0);
      let bufferData = new Float32Array(buffer);
      bufferData.set(input);
      audioData.push(bufferData);
    };
  };

  navigator.mediaDevices.getUserMedia({ audio: true, video: false }).then(recording);
}

// 録音を停止する
async function stopRecord() {
  stopRecordButton.disabled = true;
  stopRecording();
  recordProgress.innerText = "録音ファイルの作成中..."
  await fetch('/stopRecord', {
    method: "POST",
    headers: {
      'Accept': 'application/json',
      'Content-Type': 'application/octet-stream'
    },
    body: merge(audioData),
  })

  recordProgress.innerText = "作成完了"
  audioData = [];
  hideSpinner();
}

function stopRecording() {
  let track = globalStream.getTracks()[0];
  track.stop();
  input.disconnect(processor);
  processor.disconnect(context.destination);
  context.close().then(() => {
    input = null;
    processor = null;
    context = null;
    startRecordButton.disabled = false;
  });
}

function merge(data) {
  var dataLength = 0;
  for (var i = 0; i < data.length; i++) {
    dataLength += Object.keys(data[i]).length;
  }

  var result = new Float32Array(dataLength);
  var offset = 0

  for (var i = 0; i < data.length; i++) {
    result.set(data[i], offset)
    offset += Object.keys(data[i]).length
  }
  return result;
};

// 音声をテキストに変換する
async function speechToText(element) {
  showSpinner(element);
  let response = await (await fetch('/speechToText')).json();
  document.querySelector('#before-summary').value = response.message;
  hideSpinner();
}

// 要約前に文章をチェックする
function checkText() {
  const MAX_ONE_LINE = 200;
  let resultArea = document.querySelector('#check-result');
  let texts = document.querySelector('#before-summary').value.split('。').filter(item => item);
  let lineNumber = Number(document.querySelector('#line-number').value);
  let overLines = texts.filter(text => text.length > MAX_ONE_LINE);

  if (texts.length <= lineNumber) {
    return resultArea.innerText = ` 「。」区切りで、テキストは${lineNumber + 1}文以上が必要です。`;
  }
  if (overLines.length !== 0) {
    let topLabel = "次の行が、1行200文字以上となっています。¥n";
    resultArea.innerText = topLabel + overLines.map(text => '・' + text).join('。¥n');
    return;
  }
resultArea.innerText = "OK";
}

// 要約する
async function summarize(element) {
  showSpinner(element);
  const beforeSummary = document.querySelector('#before-summary').value;
  const lineNumber = document.querySelector('#line-number').value;
  const rawResponse = await fetch('/summarize', {
    method: "POST",
    headers: {
      'Accept': 'application/json',
      'Content-Type': 'application/json'
    },
    body: JSON.stringify({
      text: beforeSummary,
      lineNumber: lineNumber
    })
  })

  const content = await rawResponse.json();
  document.querySelector('#summary-area').innerText = content.message;
  hideSpinner();
}

// 要約した文を音声化する
async function textToSpeech(element) {
  showSpinner(element);
  const summary = document.querySelector('#summary-area').value;
  await fetch('/textToSpeech', {
    method: "POST",
    headers: {
      'Accept': 'application/json',
      'Content-Type': 'application/json'
    },
    body: JSON.stringify({ text: summary })
  });
  hideSpinner();
}

function showSpinner(target) {
  target.querySelector(".spinner-grow").style.display = 'inline-block';
}

function hideSpinner() {
  document.querySelectorAll('.spinner-grow').forEach(item => item.style.display = 'none');
}
